#include "gl_window_xcb.h"

#include "gl_context_xcb.h"

#include "../device_xcb.h"
#include "../window_xcb.h"

#include <cassert>

namespace saw {
xcb_gl_window::xcb_gl_window(own<xcb_window> &&win, xcb_gl_context &ctx,
							 ::GLXWindow glx_win)
	: window{std::move(win)}, context{ctx}, glx_window{glx_win} {}

xcb_gl_window::~xcb_gl_window() {
	assert(context.device);
	if (context.device) {
		::glXDestroyWindow(context.device->display, glx_window);
	}
}

void xcb_gl_window::bind() {
	assert(window && context.device && context.device->display);
	if (window) {
		if (context.device && context.device->display) {
			::glXMakeContextCurrent(context.device->display, glx_window,
									glx_window, context.context);
		}
	}
}

void xcb_gl_window::show() {
	assert(window);
	if (window) {
		window->show();
	}
}

void xcb_gl_window::hide() {
	assert(window);
	if (window) {
		window->hide();
	}
}

void xcb_gl_window::swap() {
	assert(context.device);
	assert(context.device->display);
	if (context.device && context.device->display) {
		::glXSwapBuffers(context.device->display, glx_window);
	}
}

const video_mode &xcb_gl_window::videoMode() const {
	assert(window);
	return window->videoMode();
}

const std::string_view xcb_gl_window::title() const {
	assert(window);
	if (window) {
		return window->title();
	}
	return "Bad window";
}

void xcb_gl_window::resize(size_t height, size_t width) {
	assert(window);
	if (window) {
		window->resize(height, width);
	}
}

conveyor<window::variant_event> xcb_gl_window::onevent() {
	assert(window);
	return window->onevent();
}
} // namespace saw
